@@ -2,16 +2,16 @@
 #
 # This module contains all the public symbols from the library.
 #
-import sys
-import struct
 import importlib
+import struct
+import sys
 
 ##
 #
 # Version
 #
 try:
-    from pr2modules.config.version import __version__
+    from pyroute2.config.version import __version__
 except ImportError:
     __version__ = 'unknown'
 
@@ -26,6 +26,8 @@ if sys.platform.startswith('win'):  # noqa: E402
     import win_inet_pton  # noqa: F401
 
 ##
+##
+#
 #
 # Logging setup
 #
@@ -36,11 +38,7 @@ if sys.platform.startswith('win'):  # noqa: E402
 #  * https://github.com/svinota/pyroute2/issues/573
 #  * https://github.com/svinota/pyroute2/issues/601
 #
-from pr2modules.config import log
-
-##
-#
-from pr2modules.config import entry_points_aliases
+from pyroute2.config import entry_points_aliases, log
 
 #
 #
@@ -72,11 +70,16 @@ modules = []
 namespace_inject = {}
 groups = metadata.entry_points()
 if hasattr(groups, 'select'):
-    pr2modules_group = groups.select(group='pr2modules')
+    pyroute2_group = groups.select(group='pyroute2')
 else:
-    pr2modules_group = groups.get('pr2modules', [])
-for entry_point in pr2modules_group:
-    loaded = entry_point.load()
+    pyroute2_group = groups.get('pyroute2', [])
+for entry_point in pyroute2_group:
+    try:
+        loaded = entry_point.load()
+    except ImportError:
+        if not sys.platform.startswith('win'):
+            raise
+        continue
     modules.append(entry_point.name)
     if len(entry_point.value.split(':')) == 1:
         key = 'pyroute2.%s' % entry_point.name
