@@ -1,6 +1,8 @@
 import copy
 import errno
 import queue
+import socket
+import struct
 from itertools import count
 
 from pyroute2.lab import LAB_API
@@ -13,8 +15,9 @@ from pyroute2.netlink.rtnl.rtmsg import rtmsg
 from pyroute2.requests.address import AddressFieldFilter, AddressIPRouteFilter
 from pyroute2.requests.link import LinkFieldFilter
 from pyroute2.requests.main import RequestProcessor
+from pyroute2.requests.route import RouteFieldFilter
 
-interface_counter = count(2)
+interface_counter = count(3)
 
 
 class MockLink:
@@ -35,6 +38,9 @@ class MockLink:
         kind=None,
         link=None,
         vlan_id=None,
+        master=0,
+        br_max_age=0,
+        br_forward_delay=0,
     ):
         self.index = index
         self.ifname = ifname
@@ -51,6 +57,9 @@ class MockLink:
         self.kind = kind
         self.link = link
         self.vlan_id = vlan_id
+        self.master = master
+        self.br_max_age = br_max_age
+        self.br_forward_delay = br_forward_delay
 
     def export(self):
         ret = {
@@ -207,18 +216,24 @@ class MockLink:
         }
         linkinfo = None
         infodata = {'attrs': []}
+        if self.kind is not None:
+            linkinfo = {'attrs': [('IFLA_INFO_KIND', self.kind)]}
         if self.kind not in (None, 'dummy'):
-            linkinfo = {
-                'attrs': [
-                    ('IFLA_INFO_KIND', self.kind),
-                    ('IFLA_INFO_DATA', infodata),
-                ]
-            }
+            linkinfo['attrs'].append(('IFLA_INFO_DATA', infodata))
         if self.kind == 'vlan':
             infodata['attrs'].append(('IFLA_VLAN_ID', self.vlan_id))
             ret['attrs'].append(('IFLA_LINK', self.link))
+        if self.kind == 'bridge':
+            infodata['attrs'].extend(
+                (
+                    ('IFLA_BR_MAX_AGE', self.br_max_age),
+                    ('IFLA_BR_FORWARD_DELAY', self.br_forward_delay),
+                )
+            )
         if linkinfo is not None:
             ret['attrs'].append(('IFLA_LINKINFO', linkinfo))
+        if self.master != 0:
+            ret['attrs'].append(('IFLA_MASTER', self.master))
         return ret
 
 
@@ -295,6 +310,7 @@ class MockRoute:
         scope=253,
         proto=2,
         route_type=1,
+        **kwarg,
     ):
         self.dst = dst
         self.gateway = gateway
@@ -306,17 +322,20 @@ class MockRoute:
         self.scope = scope
         self.proto = proto
         self.route_type = route_type
+        self.priority = kwarg.get('priority', 0)
+        self.tos = kwarg.get('tos', 0)
+        self._type = kwarg.get('type', 2)
 
     def export(self):
         ret = {
             'family': self.family,
             'dst_len': self.dst_len,
             'src_len': 0,
-            'tos': 0,
-            'table': self.table,
+            'tos': self.tos,
+            'table': self.table if self.table <= 255 else 252,
             'proto': self.proto,
             'scope': self.scope,
-            'type': 2,
+            'type': self._type,
             'flags': 0,
             'attrs': [('RTA_TABLE', self.table), ('RTA_OIF', self.oif)],
             'header': {
@@ -337,6 +356,8 @@ class MockRoute:
             ret['attrs'].append(('RTA_PREFSRC', self.prefsrc))
         if self.gateway is not None:
             ret['attrs'].append(('RTA_GATEWAY', self.gateway))
+        if self.priority > 0:
+            ret['attrs'].append(('RTA_PRIORITY', self.priority))
         return ret
 
 
@@ -501,6 +522,7 @@ class IPRoute(LAB_API, NetlinkSocketBase):
         keys = {
             'address': ['address', 'prefixlen', 'index', 'family'],
             'link': ['index', 'ifname'],
+            'route': ['dst', 'dst_len', 'oif', 'priority'],
         }
         check = False
         for key in keys[mode]:
@@ -513,10 +535,13 @@ class IPRoute(LAB_API, NetlinkSocketBase):
         return True
 
     def addr(self, command, **spec):
+        if command == 'dump':
+            return self.get_addr()
         request = RequestProcessor(context=spec, prime=spec)
         request.apply_filter(AddressFieldFilter())
         request.apply_filter(AddressIPRouteFilter(command))
         request.finalize()
+        address = None
 
         for address in self.preset['addr']:
             if self._match('address', address, request):
@@ -524,6 +549,8 @@ class IPRoute(LAB_API, NetlinkSocketBase):
                     raise NetlinkError(errno.EEXIST, 'address exists')
                 break
         else:
+            if command == 'del':
+                raise NetlinkError(errno.ENOENT, 'address does not exist')
             address = MockAddress(**request)
 
         if command == 'add':
@@ -537,10 +564,19 @@ class IPRoute(LAB_API, NetlinkSocketBase):
             for msg in self._get_dump([address], ifaddrmsg):
                 msg.encode()
                 self.buffer_queue.put(msg.data)
+        elif command == 'del':
+            self.preset['addr'].remove(address)
+            for msg in self._get_dump([address], ifaddrmsg):
+                msg['header']['type'] = 21
+                msg['event'] = 'RTM_DELADDR'
+                msg.encode()
+                self.buffer_queue.put(msg.data)
 
         return self._get_dump([address], ifaddrmsg)
 
     def link(self, command, **spec):
+        if command == 'dump':
+            return self.get_links()
         if 'state' in spec:
             spec['flags'] = 1 if spec.pop('state') == 'up' else 0
         request = RequestProcessor(context=spec, prime=spec)
@@ -579,6 +615,68 @@ class IPRoute(LAB_API, NetlinkSocketBase):
 
         return self._get_dump([interface], ifinfmsg)
 
+    def route(self, command, **spec):
+        if command == 'dump':
+            return self.get_routes()
+        request = RequestProcessor(context=spec, prime=spec)
+        request.apply_filter(RouteFieldFilter())
+        request.finalize()
+
+        for route in self.preset['routes']:
+            if self._match('route', route, request):
+                if command == 'add':
+                    raise NetlinkError(errno.EEXIST, 'route exists')
+                break
+        else:
+            if command == 'del':
+                raise NetlinkError(errno.ENOENT, 'route does not exist')
+            if 'tflags' in request:
+                del request['tflags']
+            if 'target' in request:
+                del request['target']
+            if 'multipath' in request:
+                del request['multipath']
+            if 'metrics' in request:
+                del request['metrics']
+            if 'deps' in request:
+                del request['deps']
+            if 'oif' not in request:
+                (gateway,) = struct.unpack(
+                    '>I', socket.inet_aton(request['gateway'])
+                )
+                for route in self.preset['routes']:
+                    if route.dst is None:
+                        continue
+                    (dst,) = struct.unpack('>I', socket.inet_aton(route.dst))
+                    if (gateway & (0xFFFFFFFF << (32 - route.dst_len))) == dst:
+                        request['oif'] = route.oif
+                        break
+                else:
+                    raise NetlinkError(errno.ENOENT, 'no route to the gateway')
+            route = MockRoute(**request)
+
+        if command == 'add':
+            self.preset['routes'].append(route)
+            for msg in self._get_dump([route], rtmsg):
+                msg.encode()
+                self.buffer_queue.put(msg.data)
+        elif command == 'set':
+            for key, value in request.items():
+                if hasattr(route, key):
+                    setattr(route, key, value)
+            for msg in self._get_dump([route], rtmsg):
+                msg.encode()
+                self.buffer_queue.put(msg.data)
+        elif command == 'del':
+            self.preset['routes'].remove(route)
+            for msg in self._get_dump([route], rtmsg):
+                msg['header']['type'] = 25
+                msg['event'] = 'RTM_DELROUTE'
+                msg.encode()
+                self.buffer_queue.put(msg.data)
+
+        return self._get_dump([route], rtmsg)
+
     def get_addr(self):
         return self._get_dump(self.preset['addr'], ifaddrmsg)
 
