@@ -213,7 +213,9 @@ class RTNL_Object(dict):
     #
     @classmethod
     def _count(cls, view):
-        return view.ndb.schema.fetchone('SELECT count(*) FROM %s' % view.table)
+        return view.ndb.task_manager.db_fetchone(
+            'SELECT count(*) FROM %s' % view.table
+        )
 
     @classmethod
     def _dump_where(cls, view):
@@ -229,7 +231,7 @@ class RTNL_Object(dict):
         )
         yield names
         where, values = cls._dump_where(view)
-        for record in view.ndb.schema.fetch(req + where, values):
+        for record in view.ndb.task_manager.db_fetch(req + where, values):
             yield record
 
     @classmethod
@@ -273,6 +275,7 @@ class RTNL_Object(dict):
         self.master = master
         self.ctxid = ctxid
         self.schema = view.ndb.schema
+        self.task_manager = view.ndb.task_manager
         self.changed = set()
         self.iclass = iclass
         self.utable = self.utable or self.table
@@ -550,7 +553,11 @@ class RTNL_Object(dict):
             # Do not trust the implicit scope and pass the
             # weakref explicitly via partial
             #
-            (self.ndb.register_handler(event, partial(wr_handler, wr, fname)))
+            (
+                self.ndb.task_manager.register_handler(
+                    event, partial(wr_handler, wr, fname)
+                )
+            )
 
     @check_auth('obj:modify')
     def snapshot(self, ctxid=None):
@@ -570,7 +577,9 @@ class RTNL_Object(dict):
         snp = type(self)(
             self.view, key, ctxid=ctxid, auth_managers=self.auth_managers
         )
-        self.ndb.schema.save_deps(ctxid, weakref.ref(snp), self.iclass)
+        self.ndb.task_manager.db_save_deps(
+            ctxid, weakref.ref(snp), self.iclass
+        )
         snp.changed = set(self.changed)
         return snp
 
@@ -615,7 +624,7 @@ class RTNL_Object(dict):
                 if value is not None and name in self.spec:
                     keys.append('f_%s = %s' % (name, self.schema.plch))
                     values.append(value)
-            spec = self.ndb.schema.fetchone(
+            spec = self.ndb.task_manager.db_fetchone(
                 'SELECT %s FROM %s WHERE %s'
                 % (' , '.join(fetch), self.etable, ' AND '.join(keys)),
                 values,
@@ -713,8 +722,8 @@ class RTNL_Object(dict):
         # variable will be saved in the traceback, so the tables will be
         # available to debug. If the traceback will be saved somewhere then
         # the tables will never be dropped by the GC, so you can do it
-        # manually by `ndb.schema.purge_snapshots()` -- to invalidate all
-        # the snapshots and to drop the associated tables.
+        # manually by `ndb.task_manager.db_purge_snapshots()` -- to invalidate
+        # all the snapshots and to drop the associated tables.
 
         self.last_save = self.snapshot()
         # Apply the changes
@@ -785,7 +794,7 @@ class RTNL_Object(dict):
             conditions.append('f_%s = %s' % (name, self.schema.plch))
             values.append(self.get(self.iclass.nla2name(name), None))
         return (
-            self.ndb.schema.fetchone(
+            self.ndb.task_manager.db_fetchone(
                 '''
                           SELECT count(*) FROM %s WHERE %s
                           '''
@@ -836,7 +845,7 @@ class RTNL_Object(dict):
 
         # Load the current state
         try:
-            self.schema.commit()
+            self.task_manager.db_commit()
         except Exception:
             pass
         self.load_sql(set_state=False)
@@ -966,7 +975,7 @@ class RTNL_Object(dict):
                     continue
                 table = cls.table
                 # comprare the tables
-                diff = self.ndb.schema.fetch(
+                diff = self.ndb.task_manager.db_fetch(
                     '''
                     SELECT * FROM %s_%s
                       EXCEPT
@@ -1030,7 +1039,7 @@ class RTNL_Object(dict):
                 self.log.debug('criteria matched')
                 return ret
             self.log.debug(f'resync the DB attempt {attempt}')
-            self.ndb.schema.flush(self['target'])
+            self.ndb.task_manager.db_flush(self['target'])
             self.load_event.clear()
             (
                 self.ndb._event_queue.put(
@@ -1105,7 +1114,7 @@ class RTNL_Object(dict):
                 value = json.dumps(value)
             values.append(value)
 
-        spec = self.ndb.schema.fetchone(
+        spec = self.ndb.task_manager.db_fetchone(
             'SELECT * FROM %s WHERE %s' % (table, ' AND '.join(keys)), values
         )
         self.log.debug('load_sql load: %s' % str(spec))
