@@ -6,17 +6,19 @@ See also: pyroute2.conntrack
 
 import socket
 
-from pyroute2.netlink import NLMSG_ERROR
-from pyroute2.netlink import NLM_F_REQUEST
-from pyroute2.netlink import NLM_F_DUMP
-from pyroute2.netlink import NLM_F_ACK
-from pyroute2.netlink import NLM_F_EXCL
-from pyroute2.netlink import NLM_F_CREATE
-from pyroute2.netlink import NETLINK_NETFILTER
-from pyroute2.netlink import nla
+from pyroute2.netlink import (
+    NETLINK_NETFILTER,
+    NLA_F_NESTED,
+    NLM_F_ACK,
+    NLM_F_CREATE,
+    NLM_F_DUMP,
+    NLM_F_EXCL,
+    NLM_F_REQUEST,
+    NLMSG_ERROR,
+    nla,
+)
+from pyroute2.netlink.nfnetlink import NFNL_SUBSYS_CTNETLINK, nfgen_msg
 from pyroute2.netlink.nlsocket import NetlinkSocket
-from pyroute2.netlink.nfnetlink import nfgen_msg
-from pyroute2.netlink.nfnetlink import NFNL_SUBSYS_CTNETLINK
 
 IPCTNL_MSG_CT_NEW = 0
 IPCTNL_MSG_CT_GET = 1
@@ -30,9 +32,11 @@ IPCTNL_MSG_MAX = 8
 
 
 try:
-    IP_PROTOCOLS = {num: name[8:]
-                    for name, num in vars(socket).items()
-                    if name.startswith("IPPROTO")}
+    IP_PROTOCOLS = {
+        num: name[8:]
+        for name, num in vars(socket).items()
+        if name.startswith("IPPROTO")
+    }
 except (IOError, OSError):
     IP_PROTOCOLS = {}
 
@@ -57,18 +61,18 @@ IP_CT_TCP_FLAG_MAXACK_SET = 0x20
 
 
 # From linux/include/net/tcp_states.h
-TCPF_ESTABLISHED = (1 << 1)
-TCPF_SYN_SENT = (1 << 2)
-TCPF_SYN_RECV = (1 << 3)
-TCPF_FIN_WAIT1 = (1 << 4)
-TCPF_FIN_WAIT2 = (1 << 5)
-TCPF_TIME_WAIT = (1 << 6)
-TCPF_CLOSE = (1 << 7)
-TCPF_CLOSE_WAIT = (1 << 8)
-TCPF_LAST_ACK = (1 << 9)
-TCPF_LISTEN = (1 << 10)
-TCPF_CLOSING = (1 << 11)
-TCPF_NEW_SYN_RECV = (1 << 12)
+TCPF_ESTABLISHED = 1 << 1
+TCPF_SYN_SENT = 1 << 2
+TCPF_SYN_RECV = 1 << 3
+TCPF_FIN_WAIT1 = 1 << 4
+TCPF_FIN_WAIT2 = 1 << 5
+TCPF_TIME_WAIT = 1 << 6
+TCPF_CLOSE = 1 << 7
+TCPF_CLOSE_WAIT = 1 << 8
+TCPF_LAST_ACK = 1 << 9
+TCPF_LISTEN = 1 << 10
+TCPF_CLOSING = 1 << 11
+TCPF_NEW_SYN_RECV = 1 << 12
 TCPF_TO_NAME = {
     TCPF_ESTABLISHED: 'ESTABLISHED',
     TCPF_SYN_SENT: 'SYN_SENT',
@@ -86,27 +90,33 @@ TCPF_TO_NAME = {
 
 
 # From include/uapi/linux/netfilter/nf_conntrack_common.h
-IPS_EXPECTED = (1 << 0)
-IPS_SEEN_REPLY = (1 << 1)
-IPS_ASSURED = (1 << 2)
-IPS_CONFIRMED = (1 << 3)
-IPS_SRC_NAT = (1 << 4)
-IPS_DST_NAT = (1 << 5)
-IPS_NAT_MASK = (IPS_DST_NAT | IPS_SRC_NAT)
-IPS_SEQ_ADJUST = (1 << 6)
-IPS_SRC_NAT_DONE = (1 << 7)
-IPS_DST_NAT_DONE = (1 << 8)
-IPS_NAT_DONE_MASK = (IPS_DST_NAT_DONE | IPS_SRC_NAT_DONE)
-IPS_DYING = (1 << 9)
-IPS_FIXED_TIMEOUT = (1 << 10)
-IPS_TEMPLATE = (1 << 11)
-IPS_UNTRACKED = (1 << 12)
-IPS_HELPER = (1 << 13)
-IPS_OFFLOAD = (1 << 14)
+IPS_EXPECTED = 1 << 0
+IPS_SEEN_REPLY = 1 << 1
+IPS_ASSURED = 1 << 2
+IPS_CONFIRMED = 1 << 3
+IPS_SRC_NAT = 1 << 4
+IPS_DST_NAT = 1 << 5
+IPS_NAT_MASK = IPS_DST_NAT | IPS_SRC_NAT
+IPS_SEQ_ADJUST = 1 << 6
+IPS_SRC_NAT_DONE = 1 << 7
+IPS_DST_NAT_DONE = 1 << 8
+IPS_NAT_DONE_MASK = IPS_DST_NAT_DONE | IPS_SRC_NAT_DONE
+IPS_DYING = 1 << 9
+IPS_FIXED_TIMEOUT = 1 << 10
+IPS_TEMPLATE = 1 << 11
+IPS_UNTRACKED = 1 << 12
+IPS_HELPER = 1 << 13
+IPS_OFFLOAD = 1 << 14
 IPS_UNCHANGEABLE_MASK = (
-    IPS_NAT_DONE_MASK | IPS_NAT_MASK |
-    IPS_EXPECTED | IPS_CONFIRMED | IPS_DYING |
-    IPS_SEQ_ADJUST | IPS_TEMPLATE | IPS_OFFLOAD)
+    IPS_NAT_DONE_MASK
+    | IPS_NAT_MASK
+    | IPS_EXPECTED
+    | IPS_CONFIRMED
+    | IPS_DYING
+    | IPS_SEQ_ADJUST
+    | IPS_TEMPLATE
+    | IPS_OFFLOAD
+)
 IPSBIT_TO_NAME = {
     IPS_EXPECTED: 'EXPECTED',
     IPS_SEEN_REPLY: 'SEEN_REPLY',
@@ -122,7 +132,7 @@ IPSBIT_TO_NAME = {
     IPS_TEMPLATE: 'TEMPLATE',
     IPS_UNTRACKED: 'UNTRACKED',
     IPS_HELPER: 'HELPER',
-    IPS_OFFLOAD: 'OFFLOAD'
+    IPS_OFFLOAD: 'OFFLOAD',
 }
 
 # From include/uapi/linux/netfilter/nf_conntrack_tcp.h
@@ -145,6 +155,38 @@ IP_CT_TCP_FLAG_TO_NAME = {
     IP_CT_TCP_SIMULTANEOUS_OPEN: 'SIMULTANEOUS_OPEN',
 }
 
+# From linux/include/uapi/linux/netfilter/nf_conntrack_tcp.h
+TCP_CONNTRACK_SYN_SENT = 1
+TCP_CONNTRACK_SYN_RECV = 2
+TCP_CONNTRACK_ESTABLISHED = 3
+TCP_CONNTRACK_FIN_WAIT = 4
+TCP_CONNTRACK_CLOSE_WAIT = 5
+TCP_CONNTRACK_LAST_ACK = 6
+TCP_CONNTRACK_TIME_WAIT = 7
+TCP_CONNTRACK_CLOSE = 8
+TCP_CONNTRACK_LISTEN = 9
+TCP_CONNTRACK_MAX = 10
+TCP_CONNTRACK_IGNORE = 11
+TCP_CONNTRACK_RETRANS = 12
+TCP_CONNTRACK_UNACK = 13
+TCP_CONNTRACK_TIMEOUT_MAX = 14
+TCP_CONNTRACK_TO_NAME = {
+    TCP_CONNTRACK_SYN_SENT: "SYN_SENT",
+    TCP_CONNTRACK_SYN_RECV: "SYN_RECV",
+    TCP_CONNTRACK_ESTABLISHED: "ESTABLISHED",
+    TCP_CONNTRACK_FIN_WAIT: "FIN_WAIT",
+    TCP_CONNTRACK_CLOSE_WAIT: "CLOSE_WAIT",
+    TCP_CONNTRACK_LAST_ACK: "LAST_ACK",
+    TCP_CONNTRACK_TIME_WAIT: "TIME_WAIT",
+    TCP_CONNTRACK_CLOSE: "CLOSE",
+    TCP_CONNTRACK_LISTEN: "LISTEN",
+    TCP_CONNTRACK_MAX: "MAX",
+    TCP_CONNTRACK_IGNORE: "IGNORE",
+    TCP_CONNTRACK_RETRANS: "RETRANS",
+    TCP_CONNTRACK_UNACK: "UNACK",
+    TCP_CONNTRACK_TIMEOUT_MAX: "TIMEOUT_MAX",
+}
+
 
 def terminate_single_msg(msg):
     return msg
@@ -209,6 +251,7 @@ class nfct_msg(nfgen_msg):
         ('CTA_LABELS', 'cta_labels'),
         ('CTA_LABELS_MASK', 'cta_labels'),
         ('CTA_SYNPROXY', 'cta_synproxy'),
+        ('CTA_FILTER', 'cta_filter'),
     )
 
     @classmethod
@@ -337,20 +380,31 @@ class nfct_msg(nfgen_msg):
             ('CTA_TIMESTAMP_STOP', 'be64'),
         )
 
+    class cta_filter(nla):
+        nla_flags = NLA_F_NESTED
+        nla_map = (
+            ('CTA_FILTER_UNSPEC', 'none'),
+            ('CTA_FILTER_ORIG_FLAGS', 'uint32'),
+            ('CTA_FILTER_REPLY_FLAGS', 'uint32'),
+        )
+
     class cta_labels(nla):
         fields = [('value', 'QQ')]
 
         def encode(self):
             if not isinstance(self['value'], tuple):
-                self['value'] = (self['value'] & 0xffffffffffffffff,
-                                 self['value'] >> 64)
+                self['value'] = (
+                    self['value'] & 0xFFFFFFFFFFFFFFFF,
+                    self['value'] >> 64,
+                )
             nla.encode(self)
 
         def decode(self):
             nla.decode(self)
             if isinstance(self['value'], tuple):
-                self['value'] = (self['value'][0] & 0xffffffffffffffff) | \
-                                (self['value'][1] << 64)
+                self['value'] = (self['value'][0] & 0xFFFFFFFFFFFFFFFF) | (
+                    self['value'][1] << 64
+                )
 
     class cta_synproxy(nla):
         nla_map = (
@@ -361,6 +415,32 @@ class nfct_msg(nfgen_msg):
         )
 
 
+FILTER_FLAG_CTA_IP_SRC = 1 << 0
+FILTER_FLAG_CTA_IP_DST = 1 << 1
+FILTER_FLAG_CTA_TUPLE_ZONE = 1 << 2
+FILTER_FLAG_CTA_PROTO_NUM = 1 << 3
+FILTER_FLAG_CTA_PROTO_SRC_PORT = 1 << 4
+FILTER_FLAG_CTA_PROTO_DST_PORT = 1 << 5
+FILTER_FLAG_CTA_PROTO_ICMP_TYPE = 1 << 6
+FILTER_FLAG_CTA_PROTO_ICMP_CODE = 1 << 7
+FILTER_FLAG_CTA_PROTO_ICMP_ID = 1 << 8
+FILTER_FLAG_CTA_PROTO_ICMPV6_TYPE = 1 << 9
+FILTER_FLAG_CTA_PROTO_ICMPV6_CODE = 1 << 10
+FILTER_FLAG_CTA_PROTO_ICMPV6_ID = 1 << 11
+
+FILTER_FLAG_ALL_CTA_PROTO = (
+    FILTER_FLAG_CTA_PROTO_SRC_PORT
+    | FILTER_FLAG_CTA_PROTO_DST_PORT
+    | FILTER_FLAG_CTA_PROTO_ICMP_TYPE
+    | FILTER_FLAG_CTA_PROTO_ICMP_CODE
+    | FILTER_FLAG_CTA_PROTO_ICMP_ID
+    | FILTER_FLAG_CTA_PROTO_ICMPV6_TYPE
+    | FILTER_FLAG_CTA_PROTO_ICMPV6_CODE
+    | FILTER_FLAG_CTA_PROTO_ICMPV6_ID
+)
+FILTER_FLAG_ALL = 0xFFFFFFFF
+
+
 class NFCTAttr(object):
     def attrs(self):
         return []
@@ -368,13 +448,31 @@ class NFCTAttr(object):
 
 class NFCTAttrTuple(NFCTAttr):
 
-    __slots__ = ('saddr', 'daddr', 'proto', 'sport', 'dport',
-                 'icmp_id', 'icmp_type', 'family', '_attr_ip',
-                 '_attr_icmp')
+    __slots__ = (
+        'saddr',
+        'daddr',
+        'proto',
+        'sport',
+        'dport',
+        'icmp_id',
+        'icmp_type',
+        'family',
+        '_attr_ip',
+        '_attr_icmp',
+    )
 
-    def __init__(self, family=socket.AF_INET,
-                 saddr=None, daddr=None, proto=None, sport=None, dport=None,
-                 icmp_id=None, icmp_type=None, icmp_code=None):
+    def __init__(
+        self,
+        family=socket.AF_INET,
+        saddr=None,
+        daddr=None,
+        proto=None,
+        sport=None,
+        dport=None,
+        icmp_id=None,
+        icmp_type=None,
+        icmp_code=None,
+    ):
         self.saddr = saddr
         self.daddr = daddr
         self.proto = proto
@@ -411,20 +509,27 @@ class NFCTAttrTuple(NFCTAttr):
         cta_proto = []
         cta_tuple = []
 
+        self.flags = 0
+
         if self.saddr is not None:
             cta_ip.append([self._attr_ip + '_SRC', self.saddr])
+            self.flags |= FILTER_FLAG_CTA_IP_SRC
 
         if self.daddr is not None:
             cta_ip.append([self._attr_ip + '_DST', self.daddr])
+            self.flags |= FILTER_FLAG_CTA_IP_DST
 
         if self.proto is not None:
             cta_proto.append(['CTA_PROTO_NUM', self.proto])
+            self.flags |= FILTER_FLAG_CTA_PROTO_NUM
 
         if self.sport is not None:
             cta_proto.append(['CTA_PROTO_SRC_PORT', self.sport])
+            self.flags |= FILTER_FLAG_CTA_PROTO_SRC_PORT
 
         if self.dport is not None:
             cta_proto.append(['CTA_PROTO_DST_PORT', self.dport])
+            self.flags |= FILTER_FLAG_CTA_PROTO_DST_PORT
 
         if self.icmp_id is not None:
             cta_proto.append([self._attr_icmp + '_ID', self.icmp_id])
@@ -504,24 +609,48 @@ class NFCTAttrTuple(NFCTAttr):
                 return False
 
             if self.proto == socket.IPPROTO_ICMP:
-                (test_attr
-                 .append((self.icmp_id, cta_proto, 'CTA_PROTO_ICMP_ID')))
-                (test_attr
-                 .append((self.icmp_type, cta_proto, 'CTA_PROTO_ICMP_TYPE')))
-                (test_attr
-                 .append((self.icmp_code, cta_proto, 'CTA_PROTO_ICMP_CODE')))
+                (
+                    test_attr.append(
+                        (self.icmp_id, cta_proto, 'CTA_PROTO_ICMP_ID')
+                    )
+                )
+                (
+                    test_attr.append(
+                        (self.icmp_type, cta_proto, 'CTA_PROTO_ICMP_TYPE')
+                    )
+                )
+                (
+                    test_attr.append(
+                        (self.icmp_code, cta_proto, 'CTA_PROTO_ICMP_CODE')
+                    )
+                )
             elif self.proto == socket.IPPROTO_ICMPV6:
-                (test_attr
-                 .append((self.icmp_id, cta_proto, 'CTA_PROTO_ICMPV6_ID')))
-                (test_attr
-                 .append((self.icmp_type, cta_proto, 'CTA_PROTO_ICMPV6_TYPE')))
-                (test_attr
-                 .append((self.icmp_code, cta_proto, 'CTA_PROTO_ICMPV6_CODE')))
+                (
+                    test_attr.append(
+                        (self.icmp_id, cta_proto, 'CTA_PROTO_ICMPV6_ID')
+                    )
+                )
+                (
+                    test_attr.append(
+                        (self.icmp_type, cta_proto, 'CTA_PROTO_ICMPV6_TYPE')
+                    )
+                )
+                (
+                    test_attr.append(
+                        (self.icmp_code, cta_proto, 'CTA_PROTO_ICMPV6_CODE')
+                    )
+                )
             elif self.proto in (socket.IPPROTO_TCP, socket.IPPROTO_UDP):
-                (test_attr
-                 .append((self.sport, cta_proto, 'CTA_PROTO_SRC_PORT')))
-                (test_attr
-                 .append((self.dport, cta_proto, 'CTA_PROTO_DST_PORT')))
+                (
+                    test_attr.append(
+                        (self.sport, cta_proto, 'CTA_PROTO_SRC_PORT')
+                    )
+                )
+                (
+                    test_attr.append(
+                        (self.dport, cta_proto, 'CTA_PROTO_DST_PORT')
+                    )
+                )
 
         for val, ndmsg, attrname in test_attr:
             if val is not None and val != ndmsg.get_attr(attrname):
@@ -572,25 +701,28 @@ class NFCTAttrTuple(NFCTAttr):
 
         r += '{}('.format(proto_name)
         if self.proto in (socket.IPPROTO_ICMP, socket.IPPROTO_ICMPV6):
-            r += 'id={}, type={}, code={}'.format(self.icmp_id,
-                                                  self.icmp_type,
-                                                  self.icmp_code)
+            r += 'id={}, type={}, code={}'.format(
+                self.icmp_id, self.icmp_type, self.icmp_code
+            )
         elif self.proto in (socket.IPPROTO_TCP, socket.IPPROTO_UDP):
             r += 'sport={}, dport={}'.format(self.sport, self.dport)
         return r + '))'
 
 
 class NFCTSocket(NetlinkSocket):
-    policy = {k | (NFNL_SUBSYS_CTNETLINK << 8): v for k, v in {
-        IPCTNL_MSG_CT_NEW: nfct_msg,
-        IPCTNL_MSG_CT_GET: nfct_msg,
-        IPCTNL_MSG_CT_DELETE: nfct_msg,
-        IPCTNL_MSG_CT_GET_CTRZERO: nfct_msg,
-        IPCTNL_MSG_CT_GET_STATS_CPU: nfct_stats_cpu,
-        IPCTNL_MSG_CT_GET_STATS: nfct_stats,
-        IPCTNL_MSG_CT_GET_DYING: nfct_msg,
-        IPCTNL_MSG_CT_GET_UNCONFIRMED: nfct_msg,
-    }.items()}
+    policy = {
+        k | (NFNL_SUBSYS_CTNETLINK << 8): v
+        for k, v in {
+            IPCTNL_MSG_CT_NEW: nfct_msg,
+            IPCTNL_MSG_CT_GET: nfct_msg,
+            IPCTNL_MSG_CT_DELETE: nfct_msg,
+            IPCTNL_MSG_CT_GET_CTRZERO: nfct_msg,
+            IPCTNL_MSG_CT_GET_STATS_CPU: nfct_stats_cpu,
+            IPCTNL_MSG_CT_GET_STATS: nfct_stats,
+            IPCTNL_MSG_CT_GET_DYING: nfct_msg,
+            IPCTNL_MSG_CT_GET_UNCONFIRMED: nfct_msg,
+        }.items()
+    }
 
     def __init__(self, nfgen_family=socket.AF_INET, **kwargs):
         super(NFCTSocket, self).__init__(family=NETLINK_NETFILTER, **kwargs)
@@ -599,33 +731,99 @@ class NFCTSocket(NetlinkSocket):
 
     def request(self, msg, msg_type, **kwargs):
         msg['nfgen_family'] = self._nfgen_family
-        msg_type |= (NFNL_SUBSYS_CTNETLINK << 8)
-        return self.nlm_request(msg, msg_type, **kwargs)
+        msg_type |= NFNL_SUBSYS_CTNETLINK << 8
+        return tuple(self.nlm_request(msg, msg_type, **kwargs))
 
-    def dump(self, mark=None, mark_mask=None):
-        msg = nfct_msg.create_from(mark=mark, mark_mask=mark_mask)
-        return self.request(msg, IPCTNL_MSG_CT_GET,
-                            msg_flags=NLM_F_REQUEST | NLM_F_DUMP)
+    def dump(
+        self,
+        mark=None,
+        mark_mask=0xFFFFFFFF,
+        tuple_orig=None,
+        tuple_reply=None,
+    ):
+        """Dump conntrack entries
+
+        Several kernel side filtering are supported:
+          * mark and mark_mask, for almost all kernel
+          * tuple_orig and tuple_reply, since kernel 5.8 and newer.
+            Warning: tuple_reply has a bug in kernel, fixed only recently.
+
+        tuple_orig and tuple_reply are type NFCTAttrTuple.
+        You can give only some attribute for filtering.
+
+        Example::
+            # Get only connections from 192.168.1.1
+            filter = NFCTAttrTuple(saddr='192.168.1.1')
+            ct.dump_entries(tuple_orig=filter)
+
+            # Get HTTPS connections
+            filter = NFCTAttrTuple(proto=socket.IPPROTO_TCP, dport=443)
+            ct.dump_entries(tuple_orig=filter)
+
+        Note that NFCTAttrTuple attributes are working like one AND operator.
+
+        Example::
+           # Get connections from 192.168.1.1 AND on port 443
+           TCP = socket.IPPROTO_TCP
+           filter = NFCTAttrTuple(saddr='192.168.1.1', proto=TCP, dport=443)
+           ct.dump_entries(tuple_orig=filter)
+
+        """
+        if tuple_orig is not None:
+            tuple_orig.attrs()  # for creating flags
+            cta_filter = {
+                'attrs': [['CTA_FILTER_ORIG_FLAGS', tuple_orig.flags]]
+            }
+            msg = nfct_msg.create_from(
+                tuple_orig=tuple_orig, cta_filter=cta_filter
+            )
+        elif tuple_reply is not None:
+            tuple_reply.attrs()
+            cta_filter = {
+                'attrs': [['CTA_FILTER_REPLY_FLAGS', tuple_reply.flags]]
+            }
+            msg = nfct_msg.create_from(
+                tuple_reply=tuple_reply, cta_filter=cta_filter
+            )
+        elif mark:
+            msg = nfct_msg.create_from(mark=mark, mark_mask=mark_mask)
+        else:
+            msg = nfct_msg.create_from()
+        return self.request(
+            msg, IPCTNL_MSG_CT_GET, msg_flags=NLM_F_REQUEST | NLM_F_DUMP
+        )
 
     def stat(self):
-        return self.request(nfct_msg(), IPCTNL_MSG_CT_GET_STATS_CPU,
-                            msg_flags=NLM_F_REQUEST | NLM_F_DUMP)
+        return self.request(
+            nfct_msg(),
+            IPCTNL_MSG_CT_GET_STATS_CPU,
+            msg_flags=NLM_F_REQUEST | NLM_F_DUMP,
+        )
 
     def count(self):
-        return self.request(nfct_msg(), IPCTNL_MSG_CT_GET_STATS,
-                            msg_flags=NLM_F_REQUEST | NLM_F_DUMP,
-                            terminate=terminate_single_msg)
+        return self.request(
+            nfct_msg(),
+            IPCTNL_MSG_CT_GET_STATS,
+            msg_flags=NLM_F_REQUEST | NLM_F_DUMP,
+            terminate=terminate_single_msg,
+        )
 
     def flush(self, mark=None, mark_mask=None):
         msg = nfct_msg.create_from(mark=mark, mark_mask=mark_mask)
-        return self.request(msg, IPCTNL_MSG_CT_DELETE,
-                            msg_flags=NLM_F_REQUEST | NLM_F_ACK,
-                            terminate=terminate_error_msg)
+        return self.request(
+            msg,
+            IPCTNL_MSG_CT_DELETE,
+            msg_flags=NLM_F_REQUEST | NLM_F_ACK,
+            terminate=terminate_error_msg,
+        )
 
     def conntrack_max_size(self):
-        return self.request(nfct_msg(), IPCTNL_MSG_CT_GET_STATS,
-                            msg_flags=NLM_F_REQUEST | NLM_F_DUMP,
-                            terminate=terminate_single_msg)
+        return self.request(
+            nfct_msg(),
+            IPCTNL_MSG_CT_GET_STATS,
+            msg_flags=NLM_F_REQUEST | NLM_F_DUMP,
+            terminate=terminate_single_msg,
+        )
 
     def entry(self, cmd, **kwargs):
         """
@@ -666,10 +864,14 @@ class NFCTSocket(NetlinkSocket):
             'del': [IPCTNL_MSG_CT_DELETE, NLM_F_ACK],
         }[cmd]
 
-        if msg_type == IPCTNL_MSG_CT_DELETE and \
-           not ('tuple_orig' in kwargs or 'tuple_reply' in kwargs):
+        if msg_type == IPCTNL_MSG_CT_DELETE and not (
+            'tuple_orig' in kwargs or 'tuple_reply' in kwargs
+        ):
             raise ValueError('Deletion requires a tuple at least')
 
-        return self.request(nfct_msg.create_from(**kwargs), msg_type,
-                            msg_flags=NLM_F_REQUEST | msg_flags,
-                            terminate=terminate_error_msg)
+        return self.request(
+            nfct_msg.create_from(**kwargs),
+            msg_type,
+            msg_flags=NLM_F_REQUEST | msg_flags,
+            terminate=terminate_error_msg,
+        )
