@@ -92,7 +92,15 @@ import time
 import traceback
 import warnings
 from functools import partial
-from socket import MSG_PEEK, SO_RCVBUF, SO_SNDBUF, SOCK_DGRAM, SOL_SOCKET
+from socket import (
+    MSG_DONTWAIT,
+    MSG_PEEK,
+    MSG_TRUNC,
+    SO_RCVBUF,
+    SO_SNDBUF,
+    SOCK_DGRAM,
+    SOL_SOCKET,
+)
 
 from pyroute2 import config
 from pyroute2.common import DEFAULT_RCVBUF, AddrPool
@@ -257,7 +265,7 @@ class Marshal:
                     pass
 
             mtype = msg['header'].get('type', None)
-            if mtype in (1, 2, 3, 4):
+            if mtype in (1, 2, 3, 4) and 'event' not in msg:
                 msg['event'] = mtypes.get(mtype, 'none')
             self.fix_message(msg)
             yield msg
@@ -331,9 +339,55 @@ class LockFactory:
         del self.locks[key]
 
 
-class NetlinkSocketBaseSafe:
+class EngineBase:
+    def __init__(self, socket):
+        self.socket = socket
+        self.get_timeout = 30
+        self.get_timeout_exception = None
+        self.change_master = threading.Event()
+        self.read_lock = threading.Lock()
+        self.qsize = 0
+
+    @property
+    def marshal(self):
+        return self.socket.marshal
+
+    @property
+    def backlog(self):
+        return self.socket.backlog
+
+    @property
+    def backlog_lock(self):
+        return self.socket.backlog_lock
+
+    @property
+    def error_deque(self):
+        return self.socket.error_deque
+
+    @property
+    def lock(self):
+        return self.socket.lock
+
+    @property
+    def buffer_queue(self):
+        return self.socket.buffer_queue
+
+    @property
+    def epid(self):
+        return self.socket.epid
+
+    @property
+    def target(self):
+        return self.socket.target
+
+    @property
+    def callbacks(self):
+        return self.socket.callbacks
+
+
+class EngineThreadSafe(EngineBase):
     '''
-    Thread-safe base class for netlink sockets. It buffers all
+    Thread-safe engine for netlink sockets. It buffers all
     incoming messages regardless sequence numbers, and returns
     only messages with requested numbers. This is done using
     synchronization primitives in a quite complicated manner.
@@ -386,7 +440,7 @@ class NetlinkSocketBaseSafe:
             msg['header']['flags'] = msg_flags
             msg['header']['sequence_number'] = msg_seq
             msg['header']['pid'] = msg_pid
-            self.sendto_gate(msg, addr)
+            self.socket.sendto_gate(msg, addr)
         except:
             raise
         finally:
@@ -447,7 +501,15 @@ class NetlinkSocketBaseSafe:
                     # Check backlog and return already collected
                     # messages.
                     #
-                    if msg_seq == 0 and self.backlog[0]:
+                    if msg_seq == -1 and any(self.backlog.values()):
+                        for seq, backlog in self.backlog.items():
+                            if backlog:
+                                for msg in backlog:
+                                    yield msg
+                                self.backlog[seq] = []
+                                enough = True
+                                break
+                    elif msg_seq == 0 and self.backlog[0]:
                         # Zero queue.
                         #
                         # Load the backlog, if there is valid
@@ -457,7 +519,7 @@ class NetlinkSocketBaseSafe:
                         self.backlog[0] = []
                         # And just exit
                         break
-                    elif msg_seq != 0 and len(self.backlog.get(msg_seq, [])):
+                    elif msg_seq > 0 and len(self.backlog.get(msg_seq, [])):
                         # Any other msg_seq.
                         #
                         # Collect messages up to the terminator.
@@ -556,10 +618,12 @@ class NetlinkSocketBaseSafe:
                                 #
                                 # This is a time consuming process, so all the
                                 # locks, except the read lock must be released
-                                data = self.recv(bufsize)
+                                data = self.socket.recv(bufsize)
                                 # Parse data
                                 msgs = tuple(
-                                    self.marshal.parse(data, msg_seq, callback)
+                                    self.socket.marshal.parse(
+                                        data, msg_seq, callback
+                                    )
                                 )
                                 # Reset ctime -- timeout should be measured
                                 # for every turn separately
@@ -644,7 +708,7 @@ class NetlinkSocketBaseSafe:
                     self.backlog_lock.release()
 
 
-class NetlinkSocketBaseUnsafe:
+class EngineThreadUnsafe(EngineBase):
     '''
     Thread unsafe nlsocket base class. Does not implement any locks
     on message processing. Discards any message if the sequence number
@@ -708,20 +772,7 @@ class NetlinkSocketBaseUnsafe:
             yield last
 
 
-class NetlinkSocketMeta(type):
-    def __new__(cls, name, bases, dct):
-        if (
-            config.nlsocket_thread_safe
-            and os.environ.get('PYROUTE2_NLSOCKET_THREAD_SAFE', 'true').lower()
-            == 'true'
-        ):
-            base = NetlinkSocketBaseSafe
-        else:
-            base = NetlinkSocketBaseUnsafe
-        return super().__new__(cls, name, bases + (base,), dct)
-
-
-class NetlinkSocketBase(metaclass=NetlinkSocketMeta):
+class NetlinkSocketBase:
     '''
     Generic netlink socket.
     '''
@@ -743,6 +794,7 @@ class NetlinkSocketBase(metaclass=NetlinkSocketMeta):
         ext_ack=False,
         strict_check=False,
         groups=0,
+        nlm_echo=False,
     ):
         # 8<-----------------------------------------
         self.config = {
@@ -759,6 +811,7 @@ class NetlinkSocketBase(metaclass=NetlinkSocketMeta):
             'ext_ack': ext_ack,
             'strict_check': strict_check,
             'groups': groups,
+            'nlm_echo': nlm_echo,
         }
         # 8<-----------------------------------------
         self.addr_pool = AddrPool(minaddr=0x000000FF, maxaddr=0x0000FFFF)
@@ -769,6 +822,7 @@ class NetlinkSocketBase(metaclass=NetlinkSocketMeta):
         self._fileno = fileno
         self._sndbuf = sndbuf
         self._rcvbuf = rcvbuf
+        self._use_peek = True
         self.backlog = {0: []}
         self.error_deque = collections.deque(maxlen=1000)
         self.callbacks = []  # [(predicate, callback, args), ...]
@@ -785,9 +839,7 @@ class NetlinkSocketBase(metaclass=NetlinkSocketMeta):
             'provide_master': config.kernel[0] > 2,
         }
         self.backlog_lock = threading.Lock()
-        self.read_lock = threading.Lock()
         self.sys_lock = threading.RLock()
-        self.change_master = threading.Event()
         self.lock = LockFactory()
         self._sock = None
         self._ctrl_read, self._ctrl_write = os.pipe()
@@ -798,10 +850,7 @@ class NetlinkSocketBase(metaclass=NetlinkSocketMeta):
             nlm_generator = config.nlm_generator
         self.nlm_generator = nlm_generator
         self.buffer_queue = Queue(maxsize=async_qsize)
-        self.qsize = 0
         self.log = []
-        self.get_timeout = 30
-        self.get_timeout_exception = None
         self.all_ns = all_ns
         self.ext_ack = ext_ack
         self.strict_check = strict_check
@@ -838,6 +887,7 @@ class NetlinkSocketBase(metaclass=NetlinkSocketMeta):
 
         # Set defaults
         self.post_init()
+        self.engine = EngineThreadSafe(self)
 
     def post_init(self):
         pass
@@ -845,6 +895,29 @@ class NetlinkSocketBase(metaclass=NetlinkSocketMeta):
     def clone(self):
         return type(self)(**self.config)
 
+    def put(
+        self,
+        msg,
+        msg_type,
+        msg_flags=NLM_F_REQUEST,
+        addr=(0, 0),
+        msg_seq=0,
+        msg_pid=None,
+    ):
+        return self.engine.put(
+            msg, msg_type, msg_flags, addr, msg_seq, msg_pid
+        )
+
+    def get(
+        self,
+        bufsize=DEFAULT_RCVBUF,
+        msg_seq=0,
+        terminate=None,
+        callback=None,
+        noraise=False,
+    ):
+        return self.engine.get(bufsize, msg_seq, terminate, callback, noraise)
+
     def close(self, code=errno.ECONNRESET):
         if code > 0 and self.input_from_buffer_queue:
             self.buffer_queue.put(
@@ -992,16 +1065,30 @@ class NetlinkSocketBase(metaclass=NetlinkSocketMeta):
 
         return ret
 
+    def _peek_bufsize(self, socket_descriptor):
+        data = bytearray()
+        try:
+            bufsize, _ = socket_descriptor.recvfrom_into(
+                data, 0, MSG_DONTWAIT | MSG_PEEK | MSG_TRUNC
+            )
+        except BlockingIOError:
+            self._use_peek = False
+            bufsize = socket_descriptor.getsockopt(SOL_SOCKET, SO_RCVBUF) // 2
+        return bufsize
+
     def sendto(self, *argv, **kwarg):
         return self._sendto(*argv, **kwarg)
 
-    def recv(self, *argv, **kwarg):
+    def recv(self, bufsize, flags=0):
         if self.input_from_buffer_queue:
             data_in = self.buffer_queue.get()
             if isinstance(data_in, Exception):
                 raise data_in
             return data_in
-        return self._sock.recv(*argv, **kwarg)
+        return self._sock.recv(
+            self._peek_bufsize(self._sock) if self._use_peek else bufsize,
+            flags,
+        )
 
     def recv_into(self, data, *argv, **kwarg):
         if self.input_from_buffer_queue:
