@@ -1,21 +1,22 @@
 import struct
-from socket import inet_ntop
-from socket import inet_pton
-from socket import AF_UNSPEC
-from socket import AF_INET
-from socket import AF_INET6
-from pyroute2.common import AF_MPLS
-from pyroute2.common import hexdump
-from pyroute2.common import map_namespace
-from pyroute2.netlink import nlmsg
-from pyroute2.netlink import nla
+from socket import AF_INET, AF_INET6, AF_UNSPEC, inet_ntop, inet_pton
+
+from pyroute2.common import AF_MPLS, hexdump, map_namespace
+from pyroute2.netlink import nla, nla_string, nlmsg
 
 RTNH_F_DEAD = 1
 RTNH_F_PERVASIVE = 2
 RTNH_F_ONLINK = 4
 RTNH_F_OFFLOAD = 8
 RTNH_F_LINKDOWN = 16
-(RTNH_F_NAMES, RTNH_F_VALUES) = map_namespace('RTNH_F', globals())
+(RTNH_F_NAMES, RTNH_F_VALUES) = map_namespace('RTNH_F_', globals())
+
+RT_SCOPE_UNIVERSE = 0
+RT_SCOPE_SITE = 200
+RT_SCOPE_LINK = 253
+RT_SCOPE_HOST = 254
+RT_SCOPE_NOWHERE = 255
+(RT_SCOPE_NAMES, RT_SCOPE_VALUES) = map_namespace('RT_SCOPE_', globals())
 
 LWTUNNEL_ENCAP_NONE = 0
 LWTUNNEL_ENCAP_MPLS = 1
@@ -26,24 +27,41 @@ LWTUNNEL_ENCAP_SEG6 = 5
 LWTUNNEL_ENCAP_BPF = 6
 LWTUNNEL_ENCAP_SEG6_LOCAL = 7
 
+IP6_RT_PRIO_USER = 1024
+
 
 class nlflags(object):
-
     def encode(self):
+        if isinstance(self['flags'], str):
+            self['flags'] = self['flags'].split(',')
         if isinstance(self['flags'], (set, tuple, list)):
             self['flags'] = self.names2flags(self['flags'])
+        if isinstance(self.get('scope'), str):
+            self['scope'] = self.name2scope(self['scope'])
         return super(nlflags, self).encode()
 
-    def flags2names(self, flags=None):
+    @staticmethod
+    def scope2name(scope):
+        return RT_SCOPE_VALUES[scope].lower()[9:]
+
+    @staticmethod
+    def name2scope(scope):
+        return RT_SCOPE_NAMES['RT_SCOPE_' + scope.upper()]
+
+    @staticmethod
+    def flags2names(flags):
         ret = []
         for flag in RTNH_F_VALUES:
             if (flag & flags) == flag:
                 ret.append(RTNH_F_VALUES[flag].lower()[7:])
         return ret
 
-    def names2flags(self, flags=None):
+    @staticmethod
+    def names2flags(flags):
         ret = 0
-        for flag in flags or self['flags']:
+        if isinstance(flags, str):
+            flags = [flags]
+        for flag in flags:
             ret |= RTNH_F_NAMES['RTNH_F_' + flag.upper()]
         return ret
 
@@ -72,46 +90,53 @@ class rtmsg_base(nlflags):
     __slots__ = ()
 
     prefix = 'RTA_'
-    sql_constraints = {'RTA_TABLE': 'NOT NULL DEFAULT 0',
-                       'RTA_DST': "NOT NULL DEFAULT ''",
-                       'RTA_PRIORITY': 'NOT NULL DEFAULT 0',
-                       'RTA_VIA': "NOT NULL DEFAULT ''",
-                       'RTA_NEWDST': "NOT NULL DEFAULT ''"}
+    sql_constraints = {
+        'RTA_TABLE': 'NOT NULL DEFAULT 0',
+        'RTA_DST': "NOT NULL DEFAULT ''",
+        'RTA_OIF': 'NOT NULL DEFAULT 0',
+        'RTA_PRIORITY': 'NOT NULL DEFAULT 0',
+        'RTA_VIA': "NOT NULL DEFAULT ''",
+        'RTA_NEWDST': "NOT NULL DEFAULT ''",
+    }
 
-    fields = (('family', 'B'),
-              ('dst_len', 'B'),
-              ('src_len', 'B'),
-              ('tos', 'B'),
-              ('table', 'B'),
-              ('proto', 'B'),
-              ('scope', 'B'),
-              ('type', 'B'),
-              ('flags', 'I'))
+    fields = (
+        ('family', 'B'),
+        ('dst_len', 'B'),
+        ('src_len', 'B'),
+        ('tos', 'B'),
+        ('table', 'B'),
+        ('proto', 'B'),
+        ('scope', 'B'),
+        ('type', 'B'),
+        ('flags', 'I'),
+    )
 
-    nla_map = (('RTA_UNSPEC', 'none'),
-               ('RTA_DST', 'target'),
-               ('RTA_SRC', 'target'),
-               ('RTA_IIF', 'uint32'),
-               ('RTA_OIF', 'uint32'),
-               ('RTA_GATEWAY', 'target'),
-               ('RTA_PRIORITY', 'uint32'),
-               ('RTA_PREFSRC', 'target'),
-               ('RTA_METRICS', 'metrics'),
-               ('RTA_MULTIPATH', '*get_nh'),
-               ('RTA_PROTOINFO', 'uint32'),
-               ('RTA_FLOW', 'uint32'),
-               ('RTA_CACHEINFO', 'cacheinfo'),
-               ('RTA_SESSION', 'hex'),
-               ('RTA_MP_ALGO', 'hex'),
-               ('RTA_TABLE', 'uint32'),
-               ('RTA_MARK', 'uint32'),
-               ('RTA_MFC_STATS', 'rta_mfc_stats'),
-               ('RTA_VIA', 'rtvia'),
-               ('RTA_NEWDST', 'target'),
-               ('RTA_PREF', 'uint8'),
-               ('RTA_ENCAP_TYPE', 'uint16'),
-               ('RTA_ENCAP', 'encap_info'),
-               ('RTA_EXPIRES', 'hex'))
+    nla_map = (
+        ('RTA_UNSPEC', 'none'),
+        ('RTA_DST', 'target'),
+        ('RTA_SRC', 'target'),
+        ('RTA_IIF', 'uint32'),
+        ('RTA_OIF', 'uint32'),
+        ('RTA_GATEWAY', 'target'),
+        ('RTA_PRIORITY', 'uint32'),
+        ('RTA_PREFSRC', 'target'),
+        ('RTA_METRICS', 'metrics'),
+        ('RTA_MULTIPATH', '*get_nh'),
+        ('RTA_PROTOINFO', 'uint32'),
+        ('RTA_FLOW', 'uint32'),
+        ('RTA_CACHEINFO', 'cacheinfo'),
+        ('RTA_SESSION', 'hex'),
+        ('RTA_MP_ALGO', 'hex'),
+        ('RTA_TABLE', 'uint32'),
+        ('RTA_MARK', 'uint32'),
+        ('RTA_MFC_STATS', 'rta_mfc_stats'),
+        ('RTA_VIA', 'rtvia'),
+        ('RTA_NEWDST', 'target'),
+        ('RTA_PREF', 'uint8'),
+        ('RTA_ENCAP_TYPE', 'uint16'),
+        ('RTA_ENCAP', 'encap_info'),
+        ('RTA_EXPIRES', 'hex'),
+    )
 
     @staticmethod
     def encap_info(self, *argv, **kwarg):
@@ -137,13 +162,16 @@ class rtmsg_base(nlflags):
                 # uint16 type
                 #
                 try:
-                    offset += struct.unpack('H', data[offset:offset + 2])[0]
+                    offset += struct.unpack('H', data[offset : offset + 2])[0]
                     # 21 == RTA_ENCAP_TYPE
                     # FIXME: should not be hardcoded
-                    if struct.unpack('H', data[offset + 2:
-                                               offset + 4])[0] == 21:
-                        encap_type = struct.unpack('H', data[offset + 4:
-                                                             offset + 6])[0]
+                    if (
+                        struct.unpack('H', data[offset + 2 : offset + 4])[0]
+                        == 21
+                    ):
+                        encap_type = struct.unpack(
+                            'H', data[offset + 4 : offset + 6]
+                        )[0]
                         break
                 except:
                     # in the case of any decoding error return self.hex
@@ -157,32 +185,38 @@ class rtmsg_base(nlflags):
         prefix = 'MPLS_IPTUNNEL_'
         __slots__ = ()
 
-        nla_map = (('MPLS_IPTUNNEL_UNSPEC', 'none'),
-                   ('MPLS_IPTUNNEL_DST', 'mpls_target'),
-                   ('MPLS_IPTUNNEL_TTL', 'uint8'))
+        nla_map = (
+            ('MPLS_IPTUNNEL_UNSPEC', 'none'),
+            ('MPLS_IPTUNNEL_DST', 'mpls_target'),
+            ('MPLS_IPTUNNEL_TTL', 'uint8'),
+        )
 
     class seg6_encap_info(nla):
 
         __slots__ = ()
 
-        nla_map = (('SEG6_IPTUNNEL_UNSPEC', 'none'),
-                   ('SEG6_IPTUNNEL_SRH', 'ipv6_sr_hdr'))
+        nla_map = (
+            ('SEG6_IPTUNNEL_UNSPEC', 'none'),
+            ('SEG6_IPTUNNEL_SRH', 'ipv6_sr_hdr'),
+        )
 
         class ipv6_sr_hdr(nla):
 
             __slots__ = ()
 
-            fields = (('encapmode', 'I'),
-                      ('nexthdr', 'B'),
-                      ('hdrlen', 'B'),
-                      ('type', 'B'),
-                      ('segments_left', 'B'),
-                      ('first_segment', 'B'),
-                      ('flags', 'B'),
-                      ('reserved', 'H'),
-                      ('segs', 's'),
-                      # Potentially several type-length-value
-                      ('tlvs', 's'))
+            fields = (
+                ('encapmode', 'I'),
+                ('nexthdr', 'B'),
+                ('hdrlen', 'B'),
+                ('type', 'B'),
+                ('segments_left', 'B'),
+                ('first_segment', 'B'),
+                ('flags', 'B'),
+                ('reserved', 'H'),
+                ('segs', 's'),
+                # Potentially several type-length-value
+                ('tlvs', 's'),
+            )
 
             # Corresponding values for seg6 encap modes
             SEG6_IPTUN_MODE_INLINE = 0
@@ -191,7 +225,7 @@ class rtmsg_base(nlflags):
             # Mapping string to nla value
             encapmodes = {
                 "inline": SEG6_IPTUN_MODE_INLINE,
-                "encap": SEG6_IPTUN_MODE_ENCAP
+                "encap": SEG6_IPTUN_MODE_ENCAP,
             }
 
             # Reverse mapping: mapping nla value to string
@@ -230,10 +264,9 @@ class rtmsg_base(nlflags):
                         # Add :: to segs
                         segs.insert(0, "::")
                     # Add mode to value
-                    self['encapmode'] = (self
-                                         .encapmodes
-                                         .get(mode,
-                                              self.SEG6_IPTUN_MODE_ENCAP))
+                    self['encapmode'] = self.encapmodes.get(
+                        mode, self.SEG6_IPTUN_MODE_ENCAP
+                    )
                     # Calculate srlen
                     srhlen = 8 + 16 * len(segs)
                     # If we are using hmac we have a tlv as trailer data
@@ -273,8 +306,9 @@ class rtmsg_base(nlflags):
                         # Put hmac
                         self['tlvs'] += struct.pack('QQQQ', 0, 0, 0, 0)
                 else:
-                    raise TypeError('Family %s not supported for seg6 tunnel'
-                                    % family)
+                    raise TypeError(
+                        'Family %s not supported for seg6 tunnel' % family
+                    )
                 # Finally encode as nla
                 nla.encode(self)
 
@@ -287,8 +321,9 @@ class rtmsg_base(nlflags):
                 # Decode the data
                 nla.decode(self)
                 # Extract the encap mode
-                self['mode'] = (self.r_encapmodes
-                                .get(self['encapmode'], "encap"))
+                self['mode'] = self.r_encapmodes.get(
+                    self['encapmode'], "encap"
+                )
                 # Calculate offset of the segs
                 offset = self.offset + 16
                 # Point the addresses
@@ -300,8 +335,9 @@ class rtmsg_base(nlflags):
                 # Move 128 bit in each step
                 for i in range(n_segs):
                     # Save the segment
-                    segs.append(inet_ntop(AF_INET6,
-                                          addresses[i * 16:i * 16 + 16]))
+                    segs.append(
+                        inet_ntop(AF_INET6, addresses[i * 16 : i * 16 + 16])
+                    )
                 # Save segs
                 self['segs'] = segs
                 # Init tlvs
@@ -309,7 +345,7 @@ class rtmsg_base(nlflags):
                 # If hmac is used
                 if self.has_hmac():
                     # Point to the start of hmac
-                    hmac = addresses[n_segs * 16:n_segs * 16 + 40]
+                    hmac = addresses[n_segs * 16 : n_segs * 16 + 40]
                     # Save tlvs section
                     self['tlvs'] = hexdump(hmac)
                     # Show also the hmac key
@@ -319,48 +355,67 @@ class rtmsg_base(nlflags):
 
         __slots__ = ()
 
-        nla_map = (('LWT_BPF_UNSPEC', 'none'),
-                   ('LWT_BPF_IN', 'bpf_obj'),
-                   ('LWT_BPF_OUT', 'bpf_obj'),
-                   ('LWT_BPF_XMIT', 'bpf_obj'),
-                   ('LWT_BPF_XMIT_HEADROOM', 'uint32'))
+        nla_map = (
+            ('LWT_BPF_UNSPEC', 'none'),
+            ('LWT_BPF_IN', 'bpf_obj'),
+            ('LWT_BPF_OUT', 'bpf_obj'),
+            ('LWT_BPF_XMIT', 'bpf_obj'),
+            ('LWT_BPF_XMIT_HEADROOM', 'uint32'),
+        )
 
         class bpf_obj(nla):
 
             __slots__ = ()
 
-            nla_map = (('LWT_BPF_PROG_UNSPEC', 'none'),
-                       ('LWT_BPF_PROG_FD', 'uint32'),
-                       ('LWT_BPF_PROG_NAME', 'asciiz'))
+            nla_map = (
+                ('LWT_BPF_PROG_UNSPEC', 'none'),
+                ('LWT_BPF_PROG_FD', 'uint32'),
+                ('LWT_BPF_PROG_NAME', 'asciiz'),
+            )
 
     class seg6local_encap_info(nla):
 
         __slots__ = ()
 
-        nla_map = (('SEG6_LOCAL_UNSPEC', 'none'),
-                   ('SEG6_LOCAL_ACTION', 'action'),
-                   ('SEG6_LOCAL_SRH', 'ipv6_sr_hdr'),
-                   ('SEG6_LOCAL_TABLE', 'table'),
-                   ('SEG6_LOCAL_NH4', 'nh4'),
-                   ('SEG6_LOCAL_NH6', 'nh6'),
-                   ('SEG6_LOCAL_IIF', 'iif'),
-                   ('SEG6_LOCAL_OIF', 'oif'),
-                   ('SEG6_LOCAL_BPF', 'none'))  # Actually not used
+        nla_map = (
+            ('SEG6_LOCAL_UNSPEC', 'none'),
+            ('SEG6_LOCAL_ACTION', 'action'),
+            ('SEG6_LOCAL_SRH', 'ipv6_sr_hdr'),
+            ('SEG6_LOCAL_TABLE', 'table'),
+            ('SEG6_LOCAL_NH4', 'nh4'),
+            ('SEG6_LOCAL_NH6', 'nh6'),
+            ('SEG6_LOCAL_IIF', 'iif'),
+            ('SEG6_LOCAL_OIF', 'oif'),
+            ('SEG6_LOCAL_BPF', 'bpf_obj'),
+            ('SEG6_LOCAL_VRFTABLE', 'vrf_table'),
+        )
+
+        class bpf_obj(nla):
+
+            __slots__ = ()
+
+            nla_map = (
+                ('LWT_BPF_PROG_UNSPEC', 'none'),
+                ('LWT_BPF_PROG_FD', 'uint32'),
+                ('LWT_BPF_PROG_NAME', 'asciiz'),
+            )
 
         class ipv6_sr_hdr(nla):
 
             __slots__ = ()
 
-            fields = (('nexthdr', 'B'),
-                      ('hdrlen', 'B'),
-                      ('type', 'B'),
-                      ('segments_left', 'B'),
-                      ('first_segment', 'B'),
-                      ('flags', 'B'),
-                      ('reserved', 'H'),
-                      ('segs', 's'),
-                      # Potentially several type-length-value
-                      ('tlvs', 's'))
+            fields = (
+                ('nexthdr', 'B'),
+                ('hdrlen', 'B'),
+                ('type', 'B'),
+                ('segments_left', 'B'),
+                ('first_segment', 'B'),
+                ('flags', 'B'),
+                ('reserved', 'H'),
+                ('segs', 's'),
+                # Potentially several type-length-value
+                ('tlvs', 's'),
+            )
 
             # Corresponding values for seg6 encap modes
             SEG6_IPTUN_MODE_INLINE = 0
@@ -369,7 +424,7 @@ class rtmsg_base(nlflags):
             # Mapping string to nla value
             encapmodes = {
                 "inline": SEG6_IPTUN_MODE_INLINE,
-                "encap": SEG6_IPTUN_MODE_ENCAP
+                "encap": SEG6_IPTUN_MODE_ENCAP,
             }
 
             # Reverse mapping: mapping nla value to string
@@ -408,10 +463,9 @@ class rtmsg_base(nlflags):
                         # Add :: to segs
                         segs.insert(0, "::")
                     # Add mode to value
-                    self['encapmode'] = (self
-                                         .encapmodes
-                                         .get(mode,
-                                              self.SEG6_IPTUN_MODE_ENCAP))
+                    self['encapmode'] = self.encapmodes.get(
+                        mode, self.SEG6_IPTUN_MODE_ENCAP
+                    )
                     # Calculate srlen
                     srhlen = 8 + 16 * len(segs)
                     # If we are using hmac we have a tlv as trailer data
@@ -451,8 +505,9 @@ class rtmsg_base(nlflags):
                         # Put hmac
                         self['tlvs'] += struct.pack('QQQQ', 0, 0, 0, 0)
                 else:
-                    raise TypeError('Family %s not supported for seg6 tunnel'
-                                    % family)
+                    raise TypeError(
+                        'Family %s not supported for seg6 tunnel' % family
+                    )
                 # Finally encode as nla
                 nla.encode(self)
 
@@ -465,8 +520,9 @@ class rtmsg_base(nlflags):
                 # Decode the data
                 nla.decode(self)
                 # Extract the encap mode
-                self['mode'] = (self.r_encapmodes
-                                .get(self['encapmode'], "encap"))
+                self['mode'] = self.r_encapmodes.get(
+                    self['encapmode'], "encap"
+                )
                 # Calculate offset of the segs
                 offset = self.offset + 16
                 # Point the addresses
@@ -478,8 +534,9 @@ class rtmsg_base(nlflags):
                 # Move 128 bit in each step
                 for i in range(n_segs):
                     # Save the segment
-                    segs.append(inet_ntop(AF_INET6,
-                                          addresses[i * 16:i * 16 + 16]))
+                    segs.append(
+                        inet_ntop(AF_INET6, addresses[i * 16 : i * 16 + 16])
+                    )
                 # Save segs
                 self['segs'] = segs
                 # Init tlvs
@@ -487,7 +544,7 @@ class rtmsg_base(nlflags):
                 # If hmac is used
                 if self.has_hmac():
                     # Point to the start of hmac
-                    hmac = addresses[n_segs * 16:n_segs * 16 + 40]
+                    hmac = addresses[n_segs * 16 : n_segs * 16 + 40]
                     # Save tlvs section
                     self['tlvs'] = hexdump(hmac)
                     # Show also the hmac key
@@ -519,32 +576,35 @@ class rtmsg_base(nlflags):
             SEG6_LOCAL_ACTION_END_AS = 13
             SEG6_LOCAL_ACTION_END_AM = 14
             SEG6_LOCAL_ACTION_END_BPF = 15
+            SEG6_LOCAL_ACTION_END_DT46 = 16
 
-            actions = {'End': SEG6_LOCAL_ACTION_END,
-                       'End.X': SEG6_LOCAL_ACTION_END_X,
-                       'End.T': SEG6_LOCAL_ACTION_END_T,
-                       'End.DX2': SEG6_LOCAL_ACTION_END_DX2,
-                       'End.DX6': SEG6_LOCAL_ACTION_END_DX6,
-                       'End.DX4': SEG6_LOCAL_ACTION_END_DX4,
-                       'End.DT6': SEG6_LOCAL_ACTION_END_DT6,
-                       'End.DT4': SEG6_LOCAL_ACTION_END_DT4,
-                       'End.B6': SEG6_LOCAL_ACTION_END_B6,
-                       'End.B6.Encaps': SEG6_LOCAL_ACTION_END_B6_ENCAP,
-                       'End.BM': SEG6_LOCAL_ACTION_END_BM,
-                       'End.S': SEG6_LOCAL_ACTION_END_S,
-                       'End.AS': SEG6_LOCAL_ACTION_END_AS,
-                       'End.AM': SEG6_LOCAL_ACTION_END_AM,
-                       'End.BPF': SEG6_LOCAL_ACTION_END_BPF}
+            actions = {
+                'End': SEG6_LOCAL_ACTION_END,
+                'End.X': SEG6_LOCAL_ACTION_END_X,
+                'End.T': SEG6_LOCAL_ACTION_END_T,
+                'End.DX2': SEG6_LOCAL_ACTION_END_DX2,
+                'End.DX6': SEG6_LOCAL_ACTION_END_DX6,
+                'End.DX4': SEG6_LOCAL_ACTION_END_DX4,
+                'End.DT6': SEG6_LOCAL_ACTION_END_DT6,
+                'End.DT4': SEG6_LOCAL_ACTION_END_DT4,
+                'End.B6': SEG6_LOCAL_ACTION_END_B6,
+                'End.B6.Encaps': SEG6_LOCAL_ACTION_END_B6_ENCAP,
+                'End.BM': SEG6_LOCAL_ACTION_END_BM,
+                'End.S': SEG6_LOCAL_ACTION_END_S,
+                'End.AS': SEG6_LOCAL_ACTION_END_AS,
+                'End.AM': SEG6_LOCAL_ACTION_END_AM,
+                'End.BPF': SEG6_LOCAL_ACTION_END_BPF,
+                'End.DT46': SEG6_LOCAL_ACTION_END_DT46,
+            }
 
             def encode(self):
                 # Get action type and convert string to value
                 action = self['value']
-                self['value'] = (self
-                                 .actions
-                                 .get(action,
-                                      self.SEG6_LOCAL_ACTION_UNSPEC))
+                self['value'] = self.actions.get(
+                    action, self.SEG6_LOCAL_ACTION_UNSPEC
+                )
                 # Convert action type to u32
-                self['value'] = self['value'] & 0xffffffff
+                self['value'] = self['value'] & 0xFFFFFFFF
                 # Finally encode as nla
                 nla.encode(self)
 
@@ -562,92 +622,100 @@ class rtmsg_base(nlflags):
             # Index of the outcoming interface
             fields = (('value', 'I'),)
 
-        class nh4(nla):
+        class nh4(nla_string):
 
             __slots__ = ()
 
             # Nexthop of the IPv4 family
-            fields = (('value', 's'),)
 
             def encode(self):
                 # Convert to network byte order
                 self['value'] = inet_pton(AF_INET, self['value'])
                 # Finally encode as nla
-                nla.encode(self)
+                nla_string.encode(self)
 
             def decode(self):
                 # Decode the data
-                nla.decode(self)
+                nla_string.decode(self)
                 # Convert the packed IP address to its string representation
-                self['value'] = inet_ntop(AF_INET,
-                                          self['value'])
+                self['value'] = inet_ntop(AF_INET, self['value'])
 
-        class nh6(nla):
+        class nh6(nla_string):
 
             __slots__ = ()
 
             # Nexthop of the IPv6 family
-            fields = (('value', 's'),)
 
             def encode(self):
                 # Convert to network byte order
                 self['value'] = inet_pton(AF_INET6, self['value'])
                 # Finally encode as nla
-                nla.encode(self)
+                nla_string.encode(self)
 
             def decode(self):
                 # Decode the data
-                nla.decode(self)
+                nla_string.decode(self)
                 # Convert the packed IP address to its string representation
                 self['value'] = inet_ntop(AF_INET6, self['value'])
 
+        class vrf_table(nla):
+
+            __slots__ = ()
+
+            # VRF Table ID
+            fields = (('value', 'I'),)
+
     #
     # TODO: add here other lwtunnel types
     #
-    encaps = {LWTUNNEL_ENCAP_MPLS: mpls_encap_info,
-              LWTUNNEL_ENCAP_SEG6: seg6_encap_info,
-              LWTUNNEL_ENCAP_BPF: bpf_encap_info,
-              LWTUNNEL_ENCAP_SEG6_LOCAL: seg6local_encap_info}
+    encaps = {
+        LWTUNNEL_ENCAP_MPLS: mpls_encap_info,
+        LWTUNNEL_ENCAP_SEG6: seg6_encap_info,
+        LWTUNNEL_ENCAP_BPF: bpf_encap_info,
+        LWTUNNEL_ENCAP_SEG6_LOCAL: seg6local_encap_info,
+    }
 
     class rta_mfc_stats(nla):
 
         __slots__ = ()
 
-        fields = (('mfcs_packets', 'uint64'),
-                  ('mfcs_bytes', 'uint64'),
-                  ('mfcs_wrong_if', 'uint64'))
+        fields = (
+            ('mfcs_packets', 'uint64'),
+            ('mfcs_bytes', 'uint64'),
+            ('mfcs_wrong_if', 'uint64'),
+        )
 
     class metrics(nla):
 
         __slots__ = ()
 
         prefix = 'RTAX_'
-        nla_map = (('RTAX_UNSPEC', 'none'),
-                   ('RTAX_LOCK', 'uint32'),
-                   ('RTAX_MTU', 'uint32'),
-                   ('RTAX_WINDOW', 'uint32'),
-                   ('RTAX_RTT', 'uint32'),
-                   ('RTAX_RTTVAR', 'uint32'),
-                   ('RTAX_SSTHRESH', 'uint32'),
-                   ('RTAX_CWND', 'uint32'),
-                   ('RTAX_ADVMSS', 'uint32'),
-                   ('RTAX_REORDERING', 'uint32'),
-                   ('RTAX_HOPLIMIT', 'uint32'),
-                   ('RTAX_INITCWND', 'uint32'),
-                   ('RTAX_FEATURES', 'uint32'),
-                   ('RTAX_RTO_MIN', 'uint32'),
-                   ('RTAX_INITRWND', 'uint32'),
-                   ('RTAX_QUICKACK', 'uint32'))
+        nla_map = (
+            ('RTAX_UNSPEC', 'none'),
+            ('RTAX_LOCK', 'uint32'),
+            ('RTAX_MTU', 'uint32'),
+            ('RTAX_WINDOW', 'uint32'),
+            ('RTAX_RTT', 'uint32'),
+            ('RTAX_RTTVAR', 'uint32'),
+            ('RTAX_SSTHRESH', 'uint32'),
+            ('RTAX_CWND', 'uint32'),
+            ('RTAX_ADVMSS', 'uint32'),
+            ('RTAX_REORDERING', 'uint32'),
+            ('RTAX_HOPLIMIT', 'uint32'),
+            ('RTAX_INITCWND', 'uint32'),
+            ('RTAX_FEATURES', 'uint32'),
+            ('RTAX_RTO_MIN', 'uint32'),
+            ('RTAX_INITRWND', 'uint32'),
+            ('RTAX_QUICKACK', 'uint32'),
+        )
 
     @staticmethod
     def get_nh(self, *argv, **kwarg):
         return nh
 
-    class rtvia(nla):
+    class rtvia(nla_string):
 
         __slots__ = ()
-
-        fields = (('value', 's'), )
         sql_type = 'TEXT'
 
         def encode(self):
@@ -655,18 +723,18 @@ class rtmsg_base(nlflags):
             if family in (AF_INET, AF_INET6):
                 addr = inet_pton(family, self['addr'])
             else:
-                raise TypeError('Family %s not supported for RTA_VIA'
-                                % family)
+                raise TypeError('Family %s not supported for RTA_VIA' % family)
             self['value'] = struct.pack('H', family) + addr
-            nla.encode(self)
+            nla_string.encode(self)
 
         def decode(self):
-            nla.decode(self)
+            nla_string.decode(self)
             family = struct.unpack('H', self['value'][:2])[0]
             addr = self['value'][2:]
-            if len(addr):
-                if (family == AF_INET and len(addr) == 4) or \
-                        (family == AF_INET6 and len(addr) == 16):
+            if addr:
+                if (family == AF_INET and len(addr) == 4) or (
+                    family == AF_INET6 and len(addr) == 16
+                ):
                     addr = inet_ntop(family, addr)
                 else:
                     addr = hexdump(addr)
@@ -676,14 +744,16 @@ class rtmsg_base(nlflags):
 
         __slots__ = ()
 
-        fields = (('rta_clntref', 'I'),
-                  ('rta_lastuse', 'I'),
-                  ('rta_expires', 'i'),
-                  ('rta_error', 'I'),
-                  ('rta_used', 'I'),
-                  ('rta_id', 'I'),
-                  ('rta_ts', 'I'),
-                  ('rta_tsage', 'I'))
+        fields = (
+            ('rta_clntref', 'I'),
+            ('rta_lastuse', 'I'),
+            ('rta_expires', 'i'),
+            ('rta_error', 'I'),
+            ('rta_used', 'I'),
+            ('rta_id', 'I'),
+            ('rta_ts', 'I'),
+            ('rta_tsage', 'I'),
+        )
 
 
 class rtmsg(rtmsg_base, nlmsg):
@@ -698,11 +768,13 @@ class rtmsg(rtmsg_base, nlmsg):
             self['type'] = 1
             # assert NLA types
             for n in self.get('attrs', []):
-                if n[0] not in ('RTA_OIF',
-                                'RTA_DST',
-                                'RTA_VIA',
-                                'RTA_NEWDST',
-                                'RTA_MULTIPATH'):
+                if n[0] not in (
+                    'RTA_OIF',
+                    'RTA_DST',
+                    'RTA_VIA',
+                    'RTA_NEWDST',
+                    'RTA_MULTIPATH',
+                ):
                     raise TypeError('Incorrect NLA type %s for AF_MPLS' % n[0])
         super(rtmsg_base, self).encode()
 
@@ -714,7 +786,5 @@ class nh(rtmsg_base, nla):
     is_nla = False
 
     sql_constraints = {}
-    cell_header = (('length', 'H'), )
-    fields = (('flags', 'B'),
-              ('hops', 'B'),
-              ('oif', 'i'))
+    cell_header = (('length', 'H'),)
+    fields = (('flags', 'B'), ('hops', 'B'), ('oif', 'i'))
